"""Settings Dialog Class"""

import logging
from PyQt5 import QtWidgets, uic
import sounddevice as sd


class Settings(QtWidgets.QDialog):
    """Settings dialog"""

    def __init__(self, WORKING_PATH, CONFIG_PATH, pref, parent=None):
        """initialize dialog"""
        super().__init__(parent)
        self.logger = logging.getLogger("__main__")
        self.config_path = CONFIG_PATH
        uic.loadUi(WORKING_PATH + "/data/configuration.ui", self)
        self.buttonBox.accepted.connect(self.save_changes)
        self.preference = pref
        self.devices = sd.query_devices()
        self.setup()

    def setup(self):
        """setup dialog"""
        for device in self.devices:
            if device.get("max_output_channels"):
                self.sounddevice.addItem(device.get("name"))
        value = self.preference.get("sounddevice", "default")
        index = self.sounddevice.findText(value)
        if index != -1:
            self.sounddevice.setCurrentIndex(index)
        self.useqrz_radioButton.setChecked(bool(self.preference.get("useqrz")))
        # self.usehamdb_radioButton.setChecked(bool(self.preference.get("usehamdb")))
        self.usehamqth_radioButton.setChecked(bool(self.preference.get("usehamqth")))
        self.lookup_user_name_field.setText(
            str(self.preference.get("lookupusername", ""))
        )
        self.lookup_password_field.setText(
            str(self.preference.get("lookuppassword", ""))
        )
        self.rigcontrolip_field.setText(str(self.preference.get("CAT_ip", "")))
        self.rigcontrolport_field.setText(str(self.preference.get("CAT_port", "")))
        self.userigctld_radioButton.setChecked(bool(self.preference.get("userigctld")))
        self.useflrig_radioButton.setChecked(bool(self.preference.get("useflrig")))

        self.cwip_field.setText(str(self.preference.get("cwip", "")))
        self.cwport_field.setText(str(self.preference.get("cwport", "")))
        self.usecwdaemon_radioButton.setChecked(
            bool(self.preference.get("cwtype") == 1)
        )
        self.usepywinkeyer_radioButton.setChecked(
            bool(self.preference.get("cwtype") == 2)
        )
        self.connect_to_server.setChecked(bool(self.preference.get("useserver")))
        self.multicast_group.setText(str(self.preference.get("multicast_group", "")))
        self.multicast_port.setText(str(self.preference.get("multicast_port", "")))
        self.interface_ip.setText(str(self.preference.get("interface_ip", "")))

        self.send_n1mm_packets.setChecked(
            bool(self.preference.get("send_n1mm_packets"))
        )
        self.n1mm_station_name.setText(
            str(self.preference.get("n1mm_station_name", ""))
        )
        self.n1mm_operator.setText(str(self.preference.get("n1mm_operator", "")))
        # self.n1mm_ip.setText(str(self.preference.get("n1mm_ip", "")))
        self.n1mm_radioport.setText(str(self.preference.get("n1mm_radioport", "")))
        self.n1mm_contactport.setText(str(self.preference.get("n1mm_contactport", "")))
        self.n1mm_lookupport.setText(str(self.preference.get("n1mm_lookupport", "")))
        self.n1mm_scoreport.setText(str(self.preference.get("n1mm_scoreport", "")))
        self.send_n1mm_radio.setChecked(bool(self.preference.get("send_n1mm_radio")))
        self.send_n1mm_contact.setChecked(
            bool(self.preference.get("send_n1mm_contact"))
        )
        self.send_n1mm_lookup.setChecked(bool(self.preference.get("send_n1mm_lookup")))
        self.send_n1mm_score.setChecked(bool(self.preference.get("send_n1mm_score")))

        self.cluster_server_field.setText(
            str(self.preference.get("cluster_server", "dxc.nc7j.com"))
        )
        self.cluster_port_field.setText(str(self.preference.get("cluster_port", 7373)))
        self.cluster_filter.setText(self.preference.get("cluster_filter", ""))
        value = self.preference.get("cluster_mode", "")
        index = self.cluster_mode.findText(value)
        if index != -1:
            self.cluster_mode.setCurrentIndex(index)
        self.activate_160m.setChecked(bool("160" in self.preference.get("bands", [])))
        self.activate_80m.setChecked(bool("80" in self.preference.get("bands", [])))
        self.activate_40m.setChecked(bool("40" in self.preference.get("bands", [])))
        self.activate_20m.setChecked(bool("20" in self.preference.get("bands", [])))
        self.activate_15m.setChecked(bool("15" in self.preference.get("bands", [])))
        self.activate_10m.setChecked(bool("10" in self.preference.get("bands", [])))
        self.activate_6m.setChecked(bool("6" in self.preference.get("bands", [])))
        self.activate_2m.setChecked(bool("2" in self.preference.get("bands", [])))
        self.activate_1dot25.setChecked(
            bool("1.25" in self.preference.get("bands", []))
        )
        self.activate_70cm.setChecked(bool("70cm" in self.preference.get("bands", [])))
        self.activate_33cm.setChecked(bool("33cm" in self.preference.get("bands", [])))
        self.activate_23cm.setChecked(bool("23cm" in self.preference.get("bands", [])))

    def save_changes(self):
        """
        Write preferences to json file.
        """
        self.preference["sounddevice"] = self.sounddevice.currentText()
        self.preference["useqrz"] = self.useqrz_radioButton.isChecked()
        # self.preference["usehamdb"] = self.usehamdb_radioButton.isChecked()
        self.preference["usehamqth"] = self.usehamqth_radioButton.isChecked()
        self.preference["lookupusername"] = self.lookup_user_name_field.text()
        self.preference["lookuppassword"] = self.lookup_password_field.text()
        self.preference["CAT_ip"] = self.rigcontrolip_field.text()
        try:
            self.preference["CAT_port"] = int(self.rigcontrolport_field.text())
        except ValueError:
            ...
        self.preference["userigctld"] = self.userigctld_radioButton.isChecked()
        self.preference["useflrig"] = self.useflrig_radioButton.isChecked()
        self.preference["cwip"] = self.cwip_field.text()
        try:
            self.preference["cwport"] = int(self.cwport_field.text())
        except ValueError:
            ...
        self.preference["cwtype"] = 0
        if self.usecwdaemon_radioButton.isChecked():
            self.preference["cwtype"] = 1
        if self.usepywinkeyer_radioButton.isChecked():
            self.preference["cwtype"] = 2
        self.preference["useserver"] = self.connect_to_server.isChecked()
        self.preference["multicast_group"] = self.multicast_group.text()
        self.preference["multicast_port"] = self.multicast_port.text()
        self.preference["interface_ip"] = self.interface_ip.text()

        self.preference["send_n1mm_packets"] = self.send_n1mm_packets.isChecked()

        self.preference["send_n1mm_radio"] = self.send_n1mm_radio.isChecked()
        self.preference["send_n1mm_contact"] = self.send_n1mm_contact.isChecked()
        self.preference["send_n1mm_lookup"] = self.send_n1mm_lookup.isChecked()
        self.preference["send_n1mm_score"] = self.send_n1mm_score.isChecked()

        self.preference["n1mm_station_name"] = self.n1mm_station_name.text()
        self.preference["n1mm_operator"] = self.n1mm_operator.text()
        # self.preference["n1mm_ip"] = self.n1mm_ip.text()
        self.preference["n1mm_radioport"] = self.n1mm_radioport.text()
        self.preference["n1mm_contactport"] = self.n1mm_contactport.text()
        self.preference["n1mm_lookupport"] = self.n1mm_lookupport.text()
        self.preference["n1mm_scoreport"] = self.n1mm_scoreport.text()
        self.preference["cluster_server"] = self.cluster_server_field.text()
        self.preference["cluster_port"] = int(self.cluster_port_field.text())
        self.preference["cluster_filter"] = self.cluster_filter.text()
        self.preference["cluster_mode"] = self.cluster_mode.currentText()
        bandlist = list()
        if self.activate_160m.isChecked():
            bandlist.append("160")
        if self.activate_80m.isChecked():
            bandlist.append("80")
        if self.activate_40m.isChecked():
            bandlist.append("40")
        if self.activate_20m.isChecked():
            bandlist.append("20")
        if self.activate_15m.isChecked():
            bandlist.append("15")
        if self.activate_10m.isChecked():
            bandlist.append("10")
        if self.activate_6m.isChecked():
            bandlist.append("6")
        if self.activate_2m.isChecked():
            bandlist.append("2")
        if self.activate_1dot25.isChecked():
            bandlist.append("1.25")
        if self.activate_70cm.isChecked():
            bandlist.append("70cm")
        if self.activate_33cm.isChecked():
            bandlist.append("33cm")
        if self.activate_23cm.isChecked():
            bandlist.append("23cm")
        self.preference["bands"] = bandlist



#!/usr/bin/env python3
"""
Check Window
"""
# pylint: disable=no-name-in-module, unused-import, no-member, invalid-name, c-extension-no-member

import logging

import platform
import queue
import os
import sys

from json import loads
from pathlib import Path

from PyQt5 import uic
from PyQt5.QtCore import QDir
from PyQt5.QtWidgets import QApplication, QListWidgetItem, QMainWindow
from PyQt5.QtGui import QFontDatabase

from not1mm.lib.database import DataBase
from not1mm.lib.multicast import Multicast
from not1mm.lib.super_check_partial import SCP

os.environ["QT_QPA_PLATFORMTHEME"] = "gnome"

WORKING_PATH = os.path.dirname(__loader__.get_filename())

if "XDG_DATA_HOME" in os.environ:
    DATA_PATH = os.environ.get("XDG_DATA_HOME")
else:
    DATA_PATH = str(Path.home() / ".local" / "share")
DATA_PATH += "/not1mm"

if "XDG_CONFIG_HOME" in os.environ:
    CONFIG_PATH = os.environ.get("XDG_CONFIG_HOME")
else:
    CONFIG_PATH = str(Path.home() / ".config")
CONFIG_PATH += "/not1mm"


class MainWindow(QMainWindow):
    """
    The main window
    """

    multicast_interface = None
    dbname = None
    pref = {}

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.load_pref()
        self.dbname = DATA_PATH + "/" + self.pref.get("current_database", "ham.db")
        self.database = DataBase(self.dbname, WORKING_PATH)
        self.database.current_contest = self.pref.get("contest", 0)
        data_path = WORKING_PATH + "/data/checkwindow.ui"
        uic.loadUi(data_path, self)
        self.setWindowTitle("CheckWindow")
        self.logList.clear()
        self.masterList.clear()
        self.telnetList.clear()
        self.callhistoryList.clear()
        self.callhistoryList.hide()
        self.callhistoryListLabel.hide()
        self.mscp = SCP(WORKING_PATH)
        self._udpwatch = None
        self.udp_fifo = queue.Queue()
        self.multicast_interface = Multicast(
            self.pref.get("multicast_group", "239.1.1.1"),
            self.pref.get("multicast_port", 2239),
            self.pref.get("interface_ip", "0.0.0.0"),
        )
        self.multicast_interface.ready_read_connect(self.watch_udp)

    def quit_app(self):
        """
        Called when the user clicks the exit button.

        Parameters
        ----------
        None

        Returns
        -------
        None
        """
        app.quit()

    def load_pref(self):
        """
        Load preference file to get current db filename.

        Parameters
        ----------
        None

        Returns
        -------
        None
        """
        try:
            if os.path.exists(CONFIG_PATH + "/not1mm.json"):
                with open(
                    CONFIG_PATH + "/not1mm.json", "rt", encoding="utf-8"
                ) as file_descriptor:
                    self.pref = loads(file_descriptor.read())
                    logger.info("%s", self.pref)
            else:
                self.pref["current_database"] = "ham.db"

        except IOError as exception:
            logger.critical("Error: %s", exception)

    def watch_udp(self):
        """
        Puts UDP datagrams in a FIFO queue.

        Parameters
        ----------
        None

        Returns
        -------
        None
        """
        while self.multicast_interface.server_udp.hasPendingDatagrams():
            json_data = self.multicast_interface.read_datagram_as_json()

            if json_data.get("station", "") != platform.node():
                continue
            if json_data.get("cmd", "") == "UPDATELOG":
                self.clear_lists()
            if json_data.get("cmd", "") == "CALLCHANGED":
                call = json_data.get("call", "")
                self.master_list(call)
                self.log_list(call)
                continue
            if json_data.get("cmd", "") == "CHECKSPOTS":
                self.telnetList.clear()
                spots = json_data.get("spots", [])
                self.telnet_list(spots)
                continue
            if json_data.get("cmd", "") == "NEWDB":
                ...
                # self.load_new_db()
            if json_data.get("cmd", "") == "HALT":
                self.quit_app()

    def clear_lists(self) -> None:
        """
        Clear match lists.

        Parameters
        ----------
        None

        Returns
        -------
        None
        """
        self.logList.clear()
        self.masterList.clear()
        self.telnetList.clear()
        self.callhistoryList.clear()

    def master_list(self, call: str) -> None:
        """
        Get MASTER.SCP matches to call and display in list.

        Parameters
        ----------
        call : str
        Call to get matches for

        Returns
        -------
        None
        """
        results = self.mscp.super_check(call)
        self.masterList.clear()
        for item in results:
            if "#" in item:
                continue
            listItem = QListWidgetItem(item)
            self.masterList.addItem(listItem)
            self.masterList.show()

    def log_list(self, call: str) -> None:
        """
        Get log matches to call and display in list.

        Parameters
        ----------
        call : str
        Call to get matches for

        Returns
        -------
        None
        """
        self.logList.clear()
        if call:
            result = self.database.get_like_calls_and_bands(call)
            for calls in result:
                listItem = QListWidgetItem(calls)
                self.logList.addItem(listItem)
                self.logList.show()

    def telnet_list(self, spots: list) -> None:
        """
        Get telnet matches to call and display in list.

        Parameters
        ----------
        spots : list
        List of spots to get matches for

        Returns
        -------
        None
        """
        self.telnetList.clear()
        if spots:
            for calls in spots:
                call = calls.get("callsign", "")
                listItem = QListWidgetItem(call)
                self.telnetList.addItem(listItem)
                self.telnetList.show()


def load_fonts_from_dir(directory: str) -> set:
    """
    Load fonts from directory.

    Parameters
    ----------
    directory : str
    The directory to load fonts from.

    Returns
    -------
    set
    The set of font families loaded.
    """
    font_families = set()
    for _fi in QDir(directory).entryInfoList(["*.ttf", "*.woff", "*.woff2"]):
        _id = QFontDatabase.addApplicationFont(_fi.absoluteFilePath())
        font_families |= set(QFontDatabase.applicationFontFamilies(_id))
    return font_families


def main():
    """main entry"""
    sys.exit(app.exec())


logger = logging.getLogger("__main__")
handler = logging.StreamHandler()
formatter = logging.Formatter(
    datefmt="%H:%M:%S",
    fmt="[%(asctime)s] %(levelname)s %(module)s - %(funcName)s Line %(lineno)d:\n%(message)s",
)
handler.setFormatter(formatter)
logger.addHandler(handler)

if Path("./debug").exists():
    logger.setLevel(logging.DEBUG)
    logger.debug("debugging on")
else:
    logger.setLevel(logging.WARNING)
    logger.warning("debugging off")

app = QApplication(sys.argv)
app.setStyle("Adwaita-Dark")
font_path = WORKING_PATH + "/data"
_families = load_fonts_from_dir(os.fspath(font_path))
window = MainWindow()
window.show()

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""Simulated Field Day club participant"""

# pylint: disable=global-statement, raise-missing-from

import random
import socket
import uuid
import time
import threading
import queue
import argparse
from random import randint
from datetime import datetime
from json import dumps, loads, JSONDecodeError


parser = argparse.ArgumentParser(description="Simulate a Field Day participant.")
parser.add_argument("-c", "--call", type=str, help="Your Callsign")
parser.add_argument("-b", "--band", type=str, help="Your Band")
parser.add_argument("-m", "--mode", type=str, help="Your Mode")
parser.add_argument("-p", "--power", type=str, help="Your Power")

args = parser.parse_args()

MULTICAST_PORT = 2239
MULTICAST_GROUP = "239.1.1.1"
INTERFACE_IP = "0.0.0.0"
GROUP_CALL = None

eightymeterstalk = (
    "What are the @stats?",
    "That K6GTE guy is a jerk!",
    "I worked your mama on 80 meters.",
    "I have nothing interesting to add, I'm just running my keys.",
    "Who's here that has gout?",
    "Jim, go to 40",
    "I gotta pee again, someone cover the GOTA station.",
    "Who made that 'Chili'... Gawd aweful!",
    ".. -..  - .- .--.  - .... .- -",
    "Why's no one covering 160?",
    "That FT8, It's so enjoyable!",
    "Yes Jim, you have to DISCONNECT the dummy load.",
)

bands = ("160", "80", "40", "20", "15", "10", "6", "2")
if args.band:
    if args.band in bands:
        BAND = args.band
    else:
        print('Allowed bands: "160", "80", "40", "20", "15", "10", "6", "2"')
        raise SystemExit(1)
else:
    BAND = bands[random.randint(0, len(bands) - 1)]

modes = ("CW", "PH", "DI")
if args.mode:
    if args.mode.upper() in modes:
        MODE = args.mode.upper()
    else:
        print('Allowed modes: "CW", "PH", "DI"')
        raise SystemExit(1)
else:
    MODE = modes[random.randint(0, len(modes) - 1)]

if args.power:
    try:
        POWER = int(args.power)
        if POWER < 1 or POWER > 100:
            raise ValueError
    except ValueError:
        print("Power is a number between 1 and 100")
        raise SystemExit(1)
else:
    POWER = 5

udp_fifo = queue.Queue()
server_commands = []


def generate_class():
    """Generates a valid Field Day class"""
    suffix = ["A", "B", "C", "D", "E", "F"][random.randint(0, 5)]
    if "C" in suffix:
        return "1C"
    if "D" in suffix:
        return "1D"
    if "E" in suffix:
        return "1E"
    if "B" in suffix:
        return str(random.randint(1, 2)) + suffix
    if "A" in suffix:
        return str(random.randint(3, 20)) + suffix

    return str(random.randint(1, 20)) + suffix


def generate_callsign():
    """Generates a US callsign, Need to add the land of maple syrup."""
    prefix = ["A", "K", "N", "W"]
    letters = [
        "A",
        "B",
        "C",
        "D",
        "E",
        "F",
        "G",
        "H",
        "I",
        "J",
        "K",
        "L",
        "M",
        "N",
        "O",
        "P",
        "Q",
        "R",
        "S",
        "T",
        "U",
        "V",
        "W",
        "X",
        "Y",
        "Z",
    ]
    callsign = prefix[random.randint(0, 3)]

    add_second_prefix_letter = random.randint(0, 2) == 0
    if "A" in callsign:  # We have no choice. Must add second prefix.
        callsign += letters[random.randint(0, 11)]
        add_second_prefix_letter = False

    if add_second_prefix_letter:
        callsign += letters[random.randint(0, 25)]

    callsign += str(random.randint(0, 9))
    if "A" in callsign[0]:
        suffix_length = random.randint(1, 2)
    else:
        length = [
            1,
            2,
            2,
            3,
            3,
            3,
        ]  # Stupid way to get a weighted result. But I'm stupid so it's normal.
        suffix_length = length[random.randint(0, 5)]

    for unused_variable in range(suffix_length):
        callsign += letters[random.randint(0, 25)]

    return callsign


def generate_section(call):
    """Generate section based on call region"""
    call_areas = {
        "0": "CO MO IA ND KS NE MN SD",
        "1": "CT RI EMA VT ME WMA NH",
        "2": "ENY NNY NLI SNJ NNJ WNY",
        "3": "DE MDC EPA WPA",
        "4": "AL SC GA SFL KY TN NC VA NFL VI PR WCF",
        "5": "AR NTX LA OK MS STX NM WTX",
        "6": "EBA SCV LAX SDG ORG SF PAC SJV SB SV",
        "7": "AK NV AZ OR EWA UT ID WWA MT WY",
        "8": "MI WV OH",
        "9": "IL WI IN",
    }
    if call[1].isdigit():
        area = call[1]
    else:
        area = call[2]
    sections = call_areas[area].split()
    return sections[random.randint(0, len(sections) - 1)]


def fakefreq(band, mode):
    """
    If unable to obtain a frequency from the rig,
    This will return a sane value for a frequency mainly for the cabrillo and adif log.
    Takes a band and mode as input and returns freq in khz.
    """
    _modes = {"CW": 0, "DI": 1, "PH": 2, "FT8": 1, "SSB": 2}
    fakefreqs = {
        "160": ["1830", "1805", "1840"],
        "80": ["3530", "3559", "3970"],
        "60": ["5332", "5373", "5405"],
        "40": ["7030", "7040", "7250"],
        "30": ["10130", "10130", "0000"],
        "20": ["14030", "14070", "14250"],
        "17": ["18080", "18100", "18150"],
        "15": ["21065", "21070", "21200"],
        "12": ["24911", "24920", "24970"],
        "10": ["28065", "28070", "28400"],
        "6": ["50.030", "50300", "50125"],
        "2": ["144030", "144144", "144250"],
        "222": ["222100", "222070", "222100"],
        "432": ["432070", "432200", "432100"],
        "SAT": ["144144", "144144", "144144"],
    }
    freqtoreturn = fakefreqs[band][_modes[mode]]
    return freqtoreturn


def log_contact():
    """Send a contgact to the server."""
    unique_id = uuid.uuid4().hex
    callsign = generate_callsign()
    contact = {
        "cmd": "POST",
        "hiscall": callsign,
        "class": generate_class(),
        "section": generate_section(callsign),
        "mode": MODE,
        "band": BAND,
        "frequency": int(float(fakefreq(BAND, MODE)) * 1000),
        "date_and_time": datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S"),
        "power": POWER,
        "grid": "DM13at",
        "opname": "John Doe",
        "station": STATION_CALL,
        "unique_id": unique_id,
    }
    server_commands.append(contact)
    bytes_to_send = bytes(dumps(contact), encoding="ascii")
    try:
        s.sendto(bytes_to_send, (MULTICAST_GROUP, int(MULTICAST_PORT)))
    except OSError as err:
        print(f"Error: {err}")
        # logging.warning("%s", err)


def remove_confirmed_commands(data):
    """Removed confirmed commands from the sent commands list."""
    for index, item in enumerate(server_commands):
        if item.get("unique_id") == data.get("unique_id") and item.get(
            "cmd"
        ) == data.get("subject"):
            server_commands.pop(index)
            print(f"Confirmed {data.get('subject')}")


def watch_udp():
    """Puts UDP datagrams in a FIFO queue"""
    while True:
        try:
            datagram = s.recv(1500)
        except socket.timeout:
            time.sleep(1)
            continue
        if datagram:
            udp_fifo.put(datagram)


def check_udp_queue():
    """checks the UDP datagram queue."""
    global GROUP_CALL
    while not udp_fifo.empty():
        datagram = udp_fifo.get()
        try:
            json_data = loads(datagram.decode())
        except UnicodeDecodeError as err:
            the_error = f"Not Unicode: {err}\n{datagram}"
            print(the_error)
            continue
        except JSONDecodeError as err:
            the_error = f"Not JSON: {err}\n{datagram}"
            print(the_error)
            continue
        # logging.info("%s", json_data)
        if json_data.get("cmd") == "PING":
            pass
            # print(f"[{strftime('%H:%M:%S', gmtime())}] {json_data}")
        if json_data.get("cmd") == "RESPONSE":
            if json_data.get("recipient") == STATION_CALL:
                if json_data.get("subject") == "HOSTINFO":
                    GROUP_CALL = str(json_data.get("groupcall"))
                    return
                if json_data.get("subject") == "LOG":
                    print("Server Generated Log.")

                remove_confirmed_commands(json_data)

        if json_data.get("cmd") == "CONFLICT":
            band, mode = json_data.get("bandmode").split()
            if (
                band == BAND
                and mode == MODE
                and json_data.get("recipient") == STATION_CALL
            ):
                print(f"CONFLICT ON {json_data.get('bandmode')}")
        if json_data.get("cmd") == "GROUPQUERY":
            if GROUP_CALL:
                send_status_udp()


def send_chat():
    """Sends UDP chat packet with text entered in chat_entry field."""
    message = eightymeterstalk[randint(0, len(eightymeterstalk) - 1)]
    packet = {"cmd": "CHAT"}
    packet["sender"] = STATION_CALL
    packet["message"] = message
    bytes_to_send = bytes(dumps(packet), encoding="ascii")
    try:
        s.sendto(bytes_to_send, (MULTICAST_GROUP, int(MULTICAST_PORT)))
    except OSError as err:
        print(f"{err}")


def query_group():
    """Sends request to server asking for group call/class/section."""
    update = {
        "cmd": "GROUPQUERY",
        "station": STATION_CALL,
    }
    bytes_to_send = bytes(dumps(update), encoding="ascii")
    try:
        s.sendto(bytes_to_send, (MULTICAST_GROUP, int(MULTICAST_PORT)))
    except OSError as err:
        print(f"{err}")


def send_status_udp():
    """Send status update to server informing of our band and mode"""

    if GROUP_CALL is None:
        query_group()
        # return

    update = {
        "cmd": "PING",
        "mode": MODE,
        "band": BAND,
        "station": STATION_CALL,
    }
    bytes_to_send = bytes(dumps(update), encoding="ascii")
    try:
        s.sendto(bytes_to_send, (MULTICAST_GROUP, int(MULTICAST_PORT)))
    except OSError as err:
        print(f"Error: {err}")


s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
s.bind(("", MULTICAST_PORT))
mreq = socket.inet_aton(MULTICAST_GROUP) + socket.inet_aton(INTERFACE_IP)
s.setsockopt(socket.IPPROTO_IP, socket.IP_ADD_MEMBERSHIP, bytes(mreq))
s.settimeout(0.01)

if args.call:
    STATION_CALL = args.call.upper()
else:
    STATION_CALL = generate_callsign()


def main():
    """The main loop"""
    _udpwatch = threading.Thread(
        target=watch_udp,
        daemon=True,
    )
    _udpwatch.start()
    print(f"Station: {STATION_CALL} on {BAND}M {MODE}")
    send_status_udp()
    count = 0
    while True:
        count += 1
        if count % 30 == 0:
            log_contact()
        if count % 15 == 0:
            send_status_udp()
        if count % 45 == 0:
            send_chat()
        check_udp_queue()

        time.sleep(1)


if __name__ == "__main__":
    main()
